/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.launcher;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput;
import org.eclipse.cosmos.rm.validation.internal.common.FileOutput;
import org.eclipse.cosmos.rm.validation.internal.common.MarkerOutput;
import org.eclipse.cosmos.rm.validation.internal.common.SystemOutput;


/**
 * This is the starting point for a validation process.  Clients are encouraged to re-use the same instance
 * with the {@link this#initialize(Map)} and/or {@link this#setAttribute(String, String)} methods 
 * to configure an instance based on a specific setting.  
 * 
 * @author Ali Mehregani
 */
public class ResourceValidation
{
	/**
	 * This attribute indicates the environment that the validation process
	 * is being done in.  The value of this attribute can be one of the following
	 * constants:<br/>
	 * 
	 * VALUE_ENV_ECLIPSE <br/>
	 * VALUE_ENV_STANDALONE <br/>
	 * Default value is VALUE_ENV_STANDALONE
	 */
	public static final String ATTRIBUTE_ENV = "attribute.environment";
		
	/**
	 * Stores the class that will be used to perform the XML validation
	 */
	public static final String ATTRIBUTE_VALDIATION_XML = "attribute.validation.xml";
	
	/**
	 * Stores the class that will be used to perform the SML (schema extension) validations
	 */
	public static final String ATTRIBUTE_VALDIATION_SML = "attribute.validation.sml";
	
	/**
	 * Stores the class that will be used to perform the schematron validation
	 */
	public static final String ATTRIBUTE_VALDIATION_SCHEMATRON = "attribute.validation.schematron";
	
	/**
	 * A path to a file which the message will be written to
	 */
	public static final String ATTRIBUTE_FILE_OUTPUT = "attribute.file.output";
	
	/**
	 * Indicates that the validation is being performed in Eclipse
	 */
	public static final String VALUE_ENV_ECLIPSE = "value.environment.eclipse";
	
	/**
	 * Indicates that the validation is being performed in standalone mode
	 */
	public static final String VALUE_ENV_STANDALONE = "value.environment.standalone";
	
	
	
	/**
	 * The attributes of this validation class
	 */
	private Map attributes;
	
	
	/**
	 * The constructor
	 */
	public ResourceValidation()
	{
		this.attributes = new Hashtable();
		setDefaultAttributes();
	}
	
	
	/**
	 * Sets default values for attributes
	 */
	protected void setDefaultAttributes()
	{
		attributes.put(ATTRIBUTE_ENV, VALUE_ENV_STANDALONE);
	}


	/**
	 * The initialize method is used to pass in multiple attributes
	 * at the same time
	 * 
	 * @param attributes The attributes.  The key and value pairs are expected 
	 * to be of type String
	 */
	public void initialize(Map attributes)
	{
		for (Iterator keys = attributes.keySet().iterator(); keys.hasNext();)
		{
			String currentKey = (String)keys.next();
			setAttribute((String)currentKey, (String)attributes.get(currentKey));
		}
	}


	/**
	 * Set an attribute of this type.  The key must be one of the ATTRIBUTE_* 
	 * constants defined by this class.
	 * 
	 * @param key The attribute key (see the ATTRIBUTE_* constants of this class)
	 * @param value The value of the attribute
	 */
	public void setAttribute(String key, String value)
	{
		attributes.put(key, value);
	}

	
	/**
	 * Start the validation process
	 */
	public void start()
	{
		/* Before we get started, we need to setup a common outstream where messages 
		 * will be logged to */
		AbstractValidationOutput validationOutput = createValidationOutput();
		IValidation xmlValidation = ValidationFactory.createValidator(IXMLValidation.class, attributes.get(ATTRIBUTE_VALDIATION_XML));
		IValidation smlValidation = ValidationFactory.createValidator(ISMLValidation.class, attributes.get(ATTRIBUTE_VALDIATION_SML));
		IValidation schematronValidation = ValidationFactory.createValidator(ISchematronValidation.class, attributes.get(ATTRIBUTE_VALDIATION_SCHEMATRON));
		
		startValidation(validationOutput, new IValidation[]{xmlValidation, smlValidation, schematronValidation});
		validationOutput.close();
	}	
	
	
	private void startValidation(AbstractValidationOutput validationOutput, IValidation[] validators)
	{
		Map attributes = new Hashtable();
		attributes.put(IValidation.ATTRIBUTE_OUTPUT, validationOutput);
		
		Object object = null;
		for (int i = 0; i < validators.length; i++)
		{
			if (validators[i] != null)
			{
				validators[i].intialize(attributes);
				object = validators[i].start(object);
			}
		}
	}


	/**
	 * Create a validation output based on the environment that the validation 
	 * is being performed in.
	 * 
	 * @return An output stream
	 */
	protected AbstractValidationOutput createValidationOutput()
	{
		String environment = (String)attributes.get(ATTRIBUTE_ENV);
		
		/* Eclipse */
		if (VALUE_ENV_ECLIPSE.equals(environment))
		{
			return new MarkerOutput();
		}
		/* Standalone */
		else if (VALUE_ENV_STANDALONE.equals(environment))
		{
			return new FileOutput();
		}
		/* Unspecified - system output by default */
		else
		{
			return new SystemOutput();
		}
	}


	/**
	 * Returns the attributes
	 * 
	 * @return attributes
	 */
	public Map getAttributes()
	{
		return attributes;
	}
}
